#ifdef GL_ES
precision highp float;
#endif

#define MODEL

attribute vec4 inPosition;
attribute vec4 inColor;
attribute vec2 inTexCoord;
attribute float inFogCoord;
attribute vec3 inNormal;

varying vec4 varColor;
varying vec2 varTexCoord;
#ifdef USE_FOG
varying float varFog;
#endif  // USE_FOG
#ifdef USE_ENV
varying vec2 varEnvCoord;
#endif  // USE_ENV

// projection matrix
uniform mat4 matViewProj;
uniform vec4 modelUpVec;
uniform vec3 cameraPos;
uniform vec3 cameraUpVec;

#ifdef USE_FOG
// fog vars
uniform bool inFog;
uniform vec2 fogParams;
#ifdef MIRROR_FOG
uniform vec3 mirrorParams;
#endif  // MIRROR_FOG
#endif  // USE_FOG

#ifdef USE_ENV
// env vars
uniform bool setEnv;
uniform bool modelEnvGood;
uniform vec3 modelEnvMatX;
uniform vec3 modelEnvMatY;
#endif  // USE_ENV

#ifdef USE_EFFECTS
// tint
uniform vec3 tint;
#endif  // USE_EFFECTS

#ifdef USE_LIGHTS
void ProcessLights(void);
#endif  // USE_LIGHTS
#ifdef USE_EFFECTS
void ProcessEffects(inout vec4 pos);
#endif  // USE_EFFECTS

void main(void)
{
  vec4 pos = inPosition;

  varColor = inColor;
  varTexCoord = inTexCoord;

  #ifdef USE_EFFECTS
  // process effects
  ProcessEffects(pos);
  #endif  // USE_EFFECTS

  // calc position
  gl_Position = pos * matViewProj;

  #ifdef USE_FOG
  // is it in fog?
  if (inFog) {
    float fogEnd = fogParams.x;
    float fogMul = fogParams.y;
    float vertFog = inFogCoord;
    #ifdef MIRROR_FOG
    float mirrorHeight = mirrorParams.x;
    float mirrorMul = mirrorParams.y;
    float mirrorAdd = mirrorParams.z;
    vertFog = dot(pos, modelUpVec);
    vertFog = mirrorHeight - vertFog;
    vertFog *= mirrorMul;
    vertFog += mirrorAdd;
    vertFog = clamp(vertFog, 0.0, 1.0);
    #endif  // MIRROR_FOG
    varFog = (fogEnd - gl_Position.w) * fogMul;
    varFog = clamp(varFog, 0.0, 1.0);
    varFog -= vertFog;
    varFog = clamp(varFog, 0.0, 1.0);
  }
  #endif  // USE_FOG

  #ifdef USE_ENV
  // calc env tex coords
  if (setEnv) {
    if (modelEnvGood) {
      vec3 vecz = normalize(pos.xyz - cameraPos);
      vec3 vecx = cross(cameraUpVec, vecz);
      vec3 vecy = cross(vecz, vecx);
      varEnvCoord.s = dot(inNormal, vecx) * 0.5 + 0.5;
      varEnvCoord.t = dot(inNormal, vecy) * 0.5 + 0.5;
    } else {
      varEnvCoord.s = dot(inNormal, modelEnvMatX) + 0.5;
      varEnvCoord.t = dot(inNormal, modelEnvMatY) + 0.5;
    }
  }
  #endif  // USE_ENV

  #ifdef USE_LIGHTS
  // process lights
  ProcessLights();
  #endif  // USE_LIGHTS

  #ifdef USE_EFFECTS
  // add any tinting
  varColor.rgb += tint;
  #endif  // USE_EFFECTS

  // clamp the color
  varColor = clamp(varColor, 0.0, 1.0);
}
